@@ -1,6 +1,8 @@
 package com.fasterxml.jackson.dataformat.xml.deser;
 
 import java.io.IOException;
+
+import javax.xml.XMLConstants;
 import javax.xml.stream.*;
 
 import org.codehaus.stax2.XMLStreamLocation2;
@@ -14,7 +16,11 @@ import com.fasterxml.jackson.core.JsonLocation;
  * abstract out all irrelevant details, and to expose equivalent of flat token
  * stream with no "fluff" tokens (comments, processing instructions, mixed
  * content) all of which is just to simplify
- * actual higher-level conversion to JSON tokens
+ * actual higher-level conversion to JSON tokens.
+ *<p>
+ * Beyond initial idea there are also couple of other detours like ability
+ * to "replay" some tokens, add virtual wrappers (ironically to support "unwrapped"
+ * array values), and to unroll "Objects" into String values in some cases.
  */
 public class XmlTokenStream
 {
@@ -25,6 +31,7 @@ public class XmlTokenStream
     public final static int XML_ATTRIBUTE_NAME = 3;
     public final static int XML_ATTRIBUTE_VALUE = 4;
     public final static int XML_TEXT = 5;
+
     public final static int XML_END = 6;
 
     // // // token replay states
@@ -32,7 +39,11 @@ public class XmlTokenStream
     private final static int REPLAY_START_DUP = 1;
     private final static int REPLAY_END = 2;
     private final static int REPLAY_START_DELAYED = 3;
-    
+
+    // Some helpful XML Constants
+
+    private final static String XSI_NAMESPACE = XMLConstants.W3C_XML_SCHEMA_INSTANCE_NS_URI;
+
     /*
     /**********************************************************************
     /* Configuration
@@ -60,6 +71,13 @@ public class XmlTokenStream
 
     protected int _attributeCount;
 
+    /**
+     * Marker used to indicate presence of `xsi:nil="true"' in current START_ELEMENT.
+     *
+     * @since 2.10
+     */
+    protected boolean _xsiNilFound;
+
     /**
      * If true we have a START_ELEMENT with mixed text
      *
@@ -72,7 +90,7 @@ public class XmlTokenStream
      * to return (as field name and value pair), if any; -1
      * when no attributes to return
      */
-    protected int _nextAttributeIndex = 0;
+    protected int _nextAttributeIndex;
 
     protected String _localName;
 
@@ -120,11 +138,12 @@ public class XmlTokenStream
                     +XMLStreamConstants.START_ELEMENT+"), instead got "+xmlReader.getEventType());
         }
         _xmlReader = Stax2ReaderAdapter.wrapIfNecessary(xmlReader);
-        _currentState = XML_START_ELEMENT;
         _localName = _xmlReader.getLocalName();
         _namespaceURI = _xmlReader.getNamespaceURI();
-        _attributeCount = _xmlReader.getAttributeCount();
         _formatFeatures = formatFeatures;
+
+        _checkXsiAttributes(); // sets _attributeCount, _nextAttributeIndex
+        _currentState = XML_START_ELEMENT;
     }
 
     public XMLStreamReader2 getXmlReader() {
@@ -146,36 +165,42 @@ public class XmlTokenStream
 
     // DEBUGGING
     /*
-    public int next() throws IOException 
+    public int next() throws XMLStreamException 
     {
         int n = next0();
         switch (n) {
         case XML_START_ELEMENT: 
-            System.out.println(" XML-token: XML_START_ELEMENT '"+_localName+"'");
+            System.out.printf(" XmlTokenStream.next(): XML_START_ELEMENT '%s' %s\n", _localName, _loc());
             break;
         case XML_END_ELEMENT: 
-            System.out.println(" XML-token: XML_END_ELEMENT '"+_localName+"'");
+            System.out.printf(" XmlTokenStream.next(): XML_END_ELEMENT '%s' %s\n", _localName, _loc());
             break;
         case XML_ATTRIBUTE_NAME: 
-            System.out.println(" XML-token: XML_ATTRIBUTE_NAME '"+_localName+"'");
+            System.out.printf(" XmlTokenStream.next(): XML_ATTRIBUTE_NAME '%s' %s\n", _localName, _loc());
             break;
         case XML_ATTRIBUTE_VALUE: 
-            System.out.println(" XML-token: XML_ATTRIBUTE_VALUE '"+_textValue+"'");
+            System.out.printf(" XmlTokenStream.next(): XML_ATTRIBUTE_VALUE '%s' %s\n", _textValue, _loc());
             break;
         case XML_TEXT: 
-            System.out.println(" XML-token: XML_TEXT '"+_textValue+"'");
+            System.out.printf(" XmlTokenStream.next(): XML_TEXT '%s' %s\n", _textValue, _loc());
             break;
         case XML_END: 
-            System.out.println(" XML-token: XML_END");
+            System.out.printf(" XmlTokenStream.next(): XML_END %s\n", _loc());
             break;
         default:
             throw new IllegalStateException();
         }
         return n;
     }
+
+    private String _loc() {
+        JsonLocation loc = getCurrentLocation();
+        return String.format("[line: %d, column: %d]", loc.getLineNr(), loc.getColumnNr());
+    }
     */
 
-    public int next() throws XMLStreamException 
+//    public int next0() throws XMLStreamException
+    public int next() throws XMLStreamException
     {
         if (_repeatElement != 0) {
             return (_currentState = _handleRepeatElement());
@@ -196,10 +221,17 @@ public class XmlTokenStream
     public String getText() { return _textValue; }
     public String getLocalName() { return _localName; }
     public String getNamespaceURI() { return _namespaceURI; }
+
+    public boolean hasXsiNil() {
+        return _xsiNilFound;
+    }
+
+    /*// not used as of 2.10
     public boolean hasAttributes() {
         return (_currentState == XML_START_ELEMENT) && (_attributeCount > 0);
     }
-    
+    */
+
     public void closeCompletely() throws XMLStreamException {
         _xmlReader.closeCompletely();
     }
@@ -225,23 +257,27 @@ public class XmlTokenStream
     /**
      * Method used to add virtual wrapping, which just duplicates START_ELEMENT
      * stream points to, and its matching closing element.
-     * 
-     * @since 2.1
      */
     protected void repeatStartElement()
     {
-//System.out.println(" -> repeatStartElement for "+_localName);        
+//System.out.println(" -> repeatStartElement for "+_localName+", _currentWrapper was: "+_currentWrapper);
         // sanity check: can only be used when just returned START_ELEMENT:
         if (_currentState != XML_START_ELEMENT) {
+            // 14-May-2020, tatu: Looks like we DO end up here with empty Lists; if so,
+            //    should NOT actually wrap.
+            if (_currentState == XML_END_ELEMENT) {
+                return;
+            }
             throw new IllegalStateException("Current state not XML_START_ELEMENT ("
                     +XML_START_ELEMENT+") but "+_currentState);
         }
         // Important: add wrapper, to keep track...
         if (_currentWrapper == null) {
-            _currentWrapper = ElementWrapper.matchingWrapper(_currentWrapper, _localName, _namespaceURI);
+            _currentWrapper = ElementWrapper.matchingWrapper(null, _localName, _namespaceURI);
         } else {
             _currentWrapper = ElementWrapper.matchingWrapper(_currentWrapper.getParent(), _localName, _namespaceURI);
         }
+//System.out.println(" repeatStartElement for "+_localName+", _currentWrapper now: "+_currentWrapper);
         _repeatElement = REPLAY_START_DUP;
     }
 
@@ -270,6 +306,10 @@ public class XmlTokenStream
         }
     }
 
+    /**
+     * Helper method called by XML String deserializer to concatenate textual contents
+     * contained in logical "Object": mostly just to skip attribute values.
+     */
     protected String convertToString() throws XMLStreamException
     {
         // only applicable to cases where START_OBJECT was induced by attributes
@@ -311,6 +351,19 @@ public class XmlTokenStream
             ++_nextAttributeIndex;
             // fall through
         case XML_START_ELEMENT: // attributes to return?
+
+            // 06-Sep-2019, tatu: `xsi:nil` to induce "real" null value?
+            if (_xsiNilFound) {
+                _xsiNilFound = false;
+                switch (_skipUntilTag()) {
+                case XMLStreamConstants.END_ELEMENT:
+                    return _handleEndElement();
+                case XMLStreamConstants.END_DOCUMENT:
+                    throw new IllegalStateException("Unexpected end-of-input after null token");
+                default:
+                }
+                throw new IllegalStateException("Unexpected START_ELEMENT after null token");
+            }
             if (_nextAttributeIndex < _attributeCount) {
                 _localName = _xmlReader.getAttributeLocalName(_nextAttributeIndex);
                 _namespaceURI = _xmlReader.getAttributeNamespace(_nextAttributeIndex);
@@ -344,17 +397,17 @@ public class XmlTokenStream
             return (_currentState = XML_ATTRIBUTE_VALUE);
         case XML_TEXT:
             // mixed text with other elements
-            if (_mixedText){
+            if (_mixedText) {
                 _mixedText = false;
                 return _initStartElement();
             }
             // text followed by END_ELEMENT
             return _handleEndElement();
+
         case XML_END:
             return XML_END;
 //            throw new IllegalStateException("No more XML tokens available (end of input)");
         }
-
         // Ok: must be END_ELEMENT; see what tag we get (or end)
         switch (_skipUntilTag()) {
         case XMLStreamConstants.END_DOCUMENT:
@@ -455,13 +508,22 @@ public class XmlTokenStream
     /* Internal methods, other
     /**********************************************************************
      */
+
+    /*
+        _xmlReader = Stax2ReaderAdapter.wrapIfNecessary(xmlReader);
+        _currentState = XML_START_ELEMENT;
+        _localName = _xmlReader.getLocalName();
+        _namespaceURI = _xmlReader.getNamespaceURI();
+        _attributeCount = _xmlReader.getAttributeCount();
+        _formatFeatures = formatFeatures;
+     */
     
     private final int _initStartElement() throws XMLStreamException
     {
         final String ns = _xmlReader.getNamespaceURI();
         final String localName = _xmlReader.getLocalName();
-        _attributeCount = _xmlReader.getAttributeCount();
-        _nextAttributeIndex = 0;
+
+        _checkXsiAttributes();
 
         /* Support for virtual wrapping: in wrapping, may either
          * create a new wrapper scope (if in sub-tree, or matches
@@ -471,12 +533,13 @@ public class XmlTokenStream
         if (_currentWrapper != null) {
             if (_currentWrapper.matchesWrapper(localName, ns)) {
                 _currentWrapper = _currentWrapper.intermediateWrapper();
+//System.out.println(" _initStartElement(): START_ELEMENT ("+localName+") DOES match ["+_currentWrapper+"]: leave/add intermediate");
             } else {
                 // implicit end is more interesting:
+//System.out.println(" _initStartElement(): START_ELEMENT ("+localName+") not matching '"+_localName+"'; add extra XML-END-ELEMENT!");
                 _localName = _currentWrapper.getWrapperLocalName();
                 _namespaceURI = _currentWrapper.getWrapperNamespace();
                 _currentWrapper = _currentWrapper.getParent();
-//System.out.println(" START_ELEMENT ("+localName+") not matching '"+_localName+"'; add extra XML-END-ELEMENT!");
                 // Important! We also need to restore the START_ELEMENT, so:
                 _nextLocalName = localName;
                 _nextNamespaceURI = ns;
@@ -489,22 +552,48 @@ public class XmlTokenStream
         return (_currentState = XML_START_ELEMENT);
     }
 
+    /**
+     * @since 2.10
+     */
+    private final void _checkXsiAttributes() {
+        int count = _xmlReader.getAttributeCount();
+        _attributeCount = count;
+
+        // [dataformat-xml#354]: xsi:nul handling; at first only if first attribute
+        if (count >= 1) {
+            if ("nil".equals(_xmlReader.getAttributeLocalName(0))) {
+                if (XSI_NAMESPACE.equals(_xmlReader.getAttributeNamespace(0))) {
+                    // need to skip, regardless of value
+                    _nextAttributeIndex = 1;
+                    // but only mark as nil marker if enabled
+                    _xsiNilFound = "true".equals(_xmlReader.getAttributeValue(0));
+                    return;
+                }
+            }
+        }
+
+        _nextAttributeIndex = 0;
+        _xsiNilFound = false;
+    }
+
     /**
      * Method called to handle details of repeating "virtual"
      * start/end elements, needed for handling 'unwrapped' lists.
      */
     protected int _handleRepeatElement() throws XMLStreamException 
     {
+//System.out.println(" XMLTokenStream._handleRepeatElement()");
+
         int type = _repeatElement;
         _repeatElement = 0;
         if (type == REPLAY_START_DUP) {
-//System.out.println("handleRepeat for START_ELEMENT: "+_localName+" ("+_xmlReader.getLocalName()+")");
+//System.out.println(" XMLTokenStream._handleRepeatElement() for START_ELEMENT: "+_localName+" ("+_xmlReader.getLocalName()+")");
             // important: add the virtual element second time, but not with name to match
             _currentWrapper = _currentWrapper.intermediateWrapper();
             return XML_START_ELEMENT;
         }
         if (type == REPLAY_END) {
-//System.out.println("handleRepeat for END_ELEMENT: "+_localName+" ("+_xmlReader.getLocalName()+")");
+//System.out.println(" XMLTokenStream._handleRepeatElement() for END_ELEMENT: "+_localName+" ("+_xmlReader.getLocalName()+")");
             _localName = _xmlReader.getLocalName();
             _namespaceURI = _xmlReader.getNamespaceURI();
             if (_currentWrapper != null) {
@@ -520,8 +609,8 @@ public class XmlTokenStream
             _namespaceURI = _nextNamespaceURI;
             _nextLocalName = null;
             _nextNamespaceURI = null;
-            
-//System.out.println("handleRepeat for START_DELAYED: "+_localName+" ("+_xmlReader.getLocalName()+")");
+
+//System.out.println(" XMLTokenStream._handleRepeatElement() for START_DELAYED: "+_localName+" ("+_xmlReader.getLocalName()+")");
 
             return XML_START_ELEMENT;
         }
@@ -530,6 +619,7 @@ public class XmlTokenStream
     
     private final int _handleEndElement()
     {
+//System.out.println(" XMLTokenStream._handleEndElement()");
         if (_currentWrapper != null) {
             ElementWrapper w = _currentWrapper;
             // important: if we close the scope, must duplicate END_ELEMENT as well
@@ -538,7 +628,7 @@ public class XmlTokenStream
                 _localName = w.getWrapperLocalName();
                 _namespaceURI = w.getWrapperNamespace();
                 _currentWrapper = _currentWrapper.getParent();
-//System.out.println(" IMPLICIT requestRepeat of END_ELEMENT '"+_localName);
+//System.out.println(" XMLTokenStream._handleEndElement(): IMPLICIT requestRepeat of END_ELEMENT '"+_localName);
             } else {
                 _currentWrapper = _currentWrapper.getParent();
             }
