@@ -18,21 +18,28 @@ public class TestNamespaces extends XmlTestBase
             this.name = name;
         }
     }
-    
+
+    static class Issue395 {
+        @JacksonXmlProperty(isAttribute = true, namespace = "http://www.w3.org/XML/1998/namespace",
+                localName = "lang")
+        public String lang = "en-US";
+    }
+
     /*
     /**********************************************************
     /* Unit tests
     /**********************************************************
      */
-    
-    // [Issue-26]: should prefer the "default namespace"
+
+    private final XmlMapper MAPPER = newMapper();
+
+    // [dataformat-xml#26]: should prefer the "default namespace"
     public void testRootNamespace() throws Exception
     {
         Person person = new Person();
         person.setName( "hello" );
         
-        XmlMapper xmlMapper = new XmlMapper();
-        String xml = xmlMapper.writeValueAsString(person);
+        String xml = MAPPER.writeValueAsString(person);
 
         // should use "the default namespace"...
         final String PREFIX = "<person xmlns=";
@@ -40,4 +47,12 @@ public class TestNamespaces extends XmlTestBase
             fail("Expected XML to begin with '"+PREFIX+"', instead got: "+xml);
         }
     }
+
+    // [dataformat-xml#395]: should not bind standard `xml` namespace to anything
+    public void testXmlNs() throws Exception
+    {
+        String xml = MAPPER.writeValueAsString(new Issue395());
+//        System.err.println("XML: "+xml);
+        assertEquals("<Issue395 xml:lang=\"en-US\"/>", xml.trim());
+    }
 }
